// TYPES

import 'dart:async';
import 'dart:math';

import 'package:flutter_query/src/notifyManager.dart';
import 'package:flutter_query/src/query_cache.dart';
import 'package:flutter_query/src/query_observer.dart';
import 'package:flutter_query/src/types.dart';
import 'package:flutter_query/src/utils.dart';

import 'utils.dart';

class QueryState {
  dynamic canFetchMore;
  dynamic data;
  dynamic error;
  int failureCount;
  bool isFetching;
  dynamic isFetchingMore;
  bool isInitialData;
  bool isInvalidated;
  QueryStatus status;
  bool throwInErrorBoundary;
  int updateCount;
  int updatedAt;

  QueryState({
    this.canFetchMore,
    this.data,
    this.error,
    this.failureCount,
    this.isFetching,
    this.isFetchingMore,
    this.isInitialData,
    this.isInvalidated,
    this.status,
    this.throwInErrorBoundary,
    this.updateCount,
    this.updatedAt,
  });
}

typedef Future RunFuture();

class FetchOptions {
  FetchMoreOptions fetchMore;

  FetchOptions({
    this.fetchMore,
  });
}

class FetchMoreOptions {
  dynamic fetchMoreVariable;
  bool previous;

  FetchMoreOptions({
    this.fetchMoreVariable,
    this.previous,
  });
}

class RefetchOptions {
  bool throwOnError;

  RefetchOptions({
    this.throwOnError,
  });
}

class SetDataOptions {
  int updatedAt;
  SetDataOptions({
    this.updatedAt,
  });
}

enum ActionType {
  Failed,
  Fetch,
  Success,
  Error,
  Invalidate,
}

class Action {
  ActionType type;
  dynamic data;
  dynamic error;
  bool canFetchMore;
  int updatedAt;
  dynamic isFetchingMore;

  Action({
    this.type,
    this.data,
    this.error,
    this.canFetchMore,
    this.isFetchingMore,
    this.updatedAt,
  });
}

class FailedAction extends Action {
  final type = ActionType.Failed;
}

class FetchAction extends Action {
  final type = ActionType.Fetch;
  // IsFetchingMoreValue isFetchingMore;
}

class SuccessAction<TResult> extends Action {
  final type = ActionType.Success;
  bool canFetchMore;
  int updatedAt;
}

class ErrorAction<TError> extends Action {
  final type = ActionType.Error;
}

class InvalidateAction extends Action {
  final type = ActionType.Invalidate;
}

// export type Action =
//   | ErrorAction<TError>
//   | FailedAction
//   | FetchAction
//   | InvalidateAction
//   | SuccessAction<TResult>

// CLASS

class Query {
  dynamic queryKey;
  String queryHash;
  ResolvedQueryConfig config;
  List<QueryObserver> observers;
  QueryState state;
  int cacheTime;

  // Rest of instance properties need to be private
  QueryCache queryCache;
  Future promise;
  Function cancelFetch;
  Function continueFetch;
  bool isTransportCancelable = false;

  Timer _gcTimer;

  Query(ResolvedQueryConfig config) {
    this.config = config;
    this.queryKey = config.queryKey;
    this.queryHash = config.queryHash;
    this.queryCache = config.queryCache;
    this.cacheTime = config.cacheTime;
    this.observers = [];
    this.state = getDefaultState(config);
    this.scheduleGc();
  }

  // private
  void updateConfig(ResolvedQueryConfig config) {
    this.config = config;
    this.cacheTime = max(this.cacheTime, config.cacheTime);
  }

  // private
  void dispatch(Action action) {
    this.state = queryReducer(this.state, action);

    notifyManager.batch(() {
      this.observers.forEach((observer) {
        observer.onQueryUpdate(action);
      });

      this.queryCache.notifyGlobalListeners(this);
    });
  }

  // private
  void scheduleGc() {
    this.clearGcTimeout();

    if (this.observers.length > 0 || !isValidTimeout(cacheTime)) {
      return;
    }

    // TODO: Get this working
    // _gcTimer = Timer(Duration(milliseconds: cacheTime), () {
    //   remove();
    // });
  }

  Future cancel([bool silent = false]) {
    final promise = this.promise;

    if (promise != null && this.cancelFetch != null) {
      this.cancelFetch(silent);
      return promise.then((_) {}).catchError((_) {});
    }

    return Future.value();
  }

  void _continue() {
    if (continueFetch != null) {
      this.continueFetch();
    }
  }

  // private
  void clearTimersObservers() {
    this.observers.forEach((observer) {
      observer.clearTimers();
    });
  }

  void clearGcTimeout() {
    if (_gcTimer != null) {
      _gcTimer.cancel();
      _gcTimer = null;
    }
  }

  void setData(
    dynamic updater, [
    SetDataOptions options,
  ]) {
    final prevData = this.state.data;

    // Get the new data
    var data = functionalUpdate(updater, prevData);

    // Structurally share data between prev and new data if needed
    if (this.config.structuralSharing ?? false) {
      // TODO: Convert this
      // data = replaceEqualDeep(prevData, data);
    }

    // Use prev data if an isDataEqual function is defined and returns `true`
    if (config.isDataEqual != null && config.isDataEqual(prevData, data)) {
      data = prevData;
    }

    // Try to determine if more data can be fetched
    final canFetchMore = hasMorePages(this.config, data);

    // Set data and mark it as cached
    this.dispatch(Action(
      type: ActionType.Success,
      data: data,
      canFetchMore: canFetchMore,
      updatedAt: options?.updatedAt,
    ));
  }

  /**
   * @deprecated
   */
  void clear() {
    print(
        'flutter-query: clear() has been deprecated, please use remove() instead');
    this.remove();
  }

  void remove() {
    this.queryCache.removeQuery(this);
  }

  void destroy() {
    this.clearGcTimeout();
    this.clearTimersObservers();
    this.cancel();
  }

  bool isActive() {
    return this.observers.any((observer) => observer.config.enabled);
  }

  bool isStale() {
    return (this.state.isInvalidated ||
        this.state.status != QueryStatus.Success ||
        this.observers.any((observer) => observer.getCurrentResult().isStale));
  }

  bool isStaleByTime([staleTime = 0]) {
    return (this.state.isInvalidated ||
        this.state.status != QueryStatus.Success ||
        this.state.updatedAt + staleTime <=
            DateTime.now().millisecondsSinceEpoch);
  }

  // TODO: We will need something similar for when a network connection comes back online
  // void onInteraction(String type) {
  //   // Execute the first observer which is enabled,
  //   // stale and wants to refetch on this interaction.
  //   final staleObserver = this.observers.firstWhere((observer) {
  //     final config = observer.config;
  //     final isStale = observer.getCurrentResult().isStale;
  //     return (
  //       config.enabled &&
  //       ((type == 'focus' &&
  //         (config.refetchOnWindowFocus == 'always' ||
  //           (config.refetchOnWindowFocus && isStale))) ||
  //         (type == 'online' &&
  //           (config.refetchOnReconnect == 'always' ||
  //             (config.refetchOnReconnect && isStale))))
  //     );
  //   });

  //   if (staleObserver != null) {
  //     staleObserver.fetch();
  //   }

  //   // Continue any paused fetch
  //   _continue();
  // }

  /**
   * @deprectated
   */
  QueryObserver subscribe(
    UpdateListener listener,
  ) {
    final observer = QueryObserver(this.config);
    observer.subscribe(listener);
    return observer;
  }

  void subscribeObserver(QueryObserver observer) {
    this.observers.add(observer);

    // Stop the query from being garbage collected
    this.clearGcTimeout();
  }

  void unsubscribeObserver(QueryObserver observer) {
    this.observers = this.observers.where((x) => x != observer).toList();

    if (this.observers.length == 0) {
      // If the transport layer does not support cancellation
      // we'll let the query continue so the result can be cached
      if (this.isTransportCancelable) {
        this.cancel();
      }

      this.scheduleGc();
    }
  }

  void invalidate() {
    if (!state.isInvalidated) {
      this.dispatch(Action(type: ActionType.Invalidate));
    }
  }

  /**
   * @deprectated
   */
  Future refetch(
    RefetchOptions options,
    ResolvedQueryConfig config,
  ) {
    var promise = this.fetch(null, config);

    if (!(options?.throwOnError ?? false)) {
      promise = promise.catchError((err) {});
    }

    return promise;
  }

  /**
   * @deprectated
   */
  Future fetchMore(
    dynamic fetchMoreVariable,
    FetchMoreOptions options,
    ResolvedQueryConfig config,
  ) {
    return this.fetch(
      FetchOptions(
        fetchMore: FetchMoreOptions(
          fetchMoreVariable: fetchMoreVariable,
          previous: options?.previous ?? false,
        ),
      ),
      config,
    );
  }

  Future fetch([
    FetchOptions options,
    ResolvedQueryConfig config,
  ]) async {
    if (this.promise != null) {
      if (options?.fetchMore != null && this.state.data != null) {
        // Silently cancel current fetch if the user wants to fetch more
        await this.cancel(true);
      } else {
        // Return current promise if we are already fetching
        return this.promise;
      }
    }

    // Update config if passed, otherwise the config from the last execution is used
    if (config != null) {
      this.updateConfig(config);
    }

    config = this.config;

    // Get the query function params
    final filter = config.queryFnParamsFilter;
    final params = filter != null ? filter(this.queryKey) : this.queryKey;
    this.promise = (() async {
      try {
        var data;

        if (config.infinite) {
          data = await this.startInfiniteFetch(config, [params], options);
        } else {
          data = await this.startFetch(config, [params], options);
        }

        // Set success state
        this.setData(data);

        // Cleanup
        this.promise = null;

        // Return data
        return data;
      } catch (error, _) {
        // Set error state if needed
        if (!(isCancelledError(error) && error.silent ?? false)) {
          this.dispatch(Action(
            type: ActionType.Error,
            error: error,
          ));
        }

        // Log error
        if (!isCancelledError(error)) {
          print(error);
        }

        // Cleanup
        this.promise = null;

        // Propagate error
        throw error;
      }
    })();

    return this.promise;
  }

  Future startFetch(
    ResolvedQueryConfig config,
    dynamic params,
    FetchOptions _options,
  ) {
    // Create function to fetch the data
    // TODO: Convert this (refer to react-query source)
    final fetchData = () => config.queryFn();

    // Set to fetching state if not already in it
    if (!this.state.isFetching) {
      this.dispatch(Action(type: ActionType.Fetch));
    }
    // Try to fetch the data
    return this.tryFetchData(config, fetchData);
  }

  Future startInfiniteFetch(
    ResolvedQueryConfig config,
    List params,
    FetchOptions options,
  ) {
    final fetchMore = options?.fetchMore;
    final previous = fetchMore?.previous ?? false;
    final fetchMoreVariable = fetchMore?.fetchMoreVariable;
    final isFetchingMore =
        fetchMore != null ? (previous ? 'previous' : 'next') : false;
    final prevPages = (this.state.data as dynamic) ?? [];

    // Create function to fetch a page
    final fetchPage = (
      List pages, [
      bool prepend = false,
      dynamic cursor,
    ]) async {
      final lastPage = getLastPage(pages, prepend);

      if (cursor == null && lastPage != null && config.getFetchMore != null) {
        cursor = config.getFetchMore(lastPage, pages);
      }

      if ((cursor == null || cursor == false) && lastPage != null) {
        return pages;
      }

      // TODO: Convert this (Refer to react-query source)
      final page = await config.queryFn(cursor);

      return prepend ? [page, ...pages] : [...pages, page];
    };

    // Create function to fetch the data
    final fetchData = () {
      // TODO: Simplify this
      if ((isFetchingMore is bool && isFetchingMore) ||
          (!(isFetchingMore is bool) && isFetchingMore != null)) {
        // TODO: Refer to the react query source and make sure this conversion is valid
        return fetchPage(prevPages ?? [], previous, fetchMoreVariable);
      } else if (prevPages.length == 0) {
        return fetchPage([]);
      } else {
        var promise = fetchPage([]);
        for (var i = 1; i < prevPages.length; i++) {
          promise = promise.then((d) => fetchPage(d));
        }
        return promise;
      }
    };

    // Set to fetching state if not already in it
    if (!this.state.isFetching || this.state.isFetchingMore != isFetchingMore) {
      this.dispatch(
        Action(
          type: ActionType.Fetch,
          // TODO: Refer to the react query source and make sure this conversion is valid
          isFetchingMore: isFetchingMore,
        ),
      );
    }

    // Try to get the data
    return this.tryFetchData(config, fetchData);
  }

  Future tryFetchData(
    ResolvedQueryConfig config,
    Function fn,
  ) {
    var resolved = false;
    Function continueLoop;
    Function cancelTransport;

    final done = () {
      resolved = true;

      this.cancelFetch = null;
      this.continueFetch = null;
      this.isTransportCancelable = false;

      // End loop if currently paused
      if (continueLoop != null) {
        continueLoop();
      }
    };

    // Create callback to cancel this fetch
    this.cancelFetch = (silent) {
      if (cancelTransport != null) {
        cancelTransport();
      }

      throw CancelledError(silent);
    };

    // Create callback to continue this fetch
    this.continueFetch = () {
      if (continueLoop != null) {
        continueLoop();
      }
    };

    RunFuture run;
    // Create loop function
    run = () async {
      try {
        // Execute query
        final promiseOrValue = fn();

        // Check if the transport layer support cancellation
        if (isCancelable(promiseOrValue)) {
          cancelTransport = () {
            try {
              promiseOrValue.cancel();
            } catch (e) {}
          };
          this.isTransportCancelable = true;
        }

        // Await data
        return promiseOrValue is Future ? await promiseOrValue : promiseOrValue;
      } catch (error, _) {
        // Stop if the fetch is already resolved
        if (resolved) {
          return;
        }

        // Do we need to retry the request?
        final failureCount = this.state.failureCount;
        final retry = config.retry;
        final retryDelay = config.retryDelay;

        final shouldRetry = retry == true ||
            failureCount < retry ||
            (retry is Function && retry(failureCount, error));

        if (!shouldRetry) {
          // We are done if the query does not need to be retried
          throw error;
        }

        // Increase the failureCount
        this.dispatch(Action(type: ActionType.Failed));

        // Delay
        await sleep(functionalUpdate(retryDelay, failureCount) ?? 0);

        // Pause retry if the document is not visible or when the device is offline
        // TODO: Convert this
        // if (!isDocumentVisible() || !isOnline()) {
        //   await Future((continueResolve) {
        //     continueLoop = continueResolve;
        //   });
        // }

        // Try again if not resolved yet
        if (!resolved) {
          return run();
        }
      }
    };

    // Start loop
    return run().then((value) {
      done();
      return value;
    }).catchError((err) {
      done();
      throw err;
    });
  }
}

dynamic getLastPage(List pages, bool previous) {
  if (pages != null && pages.length > 0) {
    return previous ? pages[0] : pages[pages.length - 1];
  } else {
    return null;
  }
}

bool hasMorePages(
  ResolvedQueryConfig config,
  dynamic pages, [
  bool previous = false,
]) {
  if (config.infinite && config.getFetchMore != null && pages is List) {
    final cursor = config.getFetchMore(getLastPage(pages, previous), pages);
    return cursor != null && cursor != false;
  }
  return false;
}

QueryState getDefaultState(ResolvedQueryConfig config) {
  final data = config.initialData is Function
      ? config.initialData()
      : config.initialData;

  final status = data != null
      ? QueryStatus.Success
      : config.enabled
          ? QueryStatus.Loading
          : QueryStatus.Idle;

  return QueryState(
    canFetchMore: hasMorePages(config, data),
    data: data,
    error: null,
    failureCount: 0,
    isFetching: status == QueryStatus.Loading,
    isFetchingMore: false,
    isInitialData: true,
    isInvalidated: false,
    status: status,
    updateCount: 0,
    updatedAt: DateTime.now().millisecondsSinceEpoch,
  );
}

QueryState queryReducer(
  QueryState state,
  Action action,
) {
  state = state ?? QueryState();
  // TODO: Make this immutable!!!!!
  switch (action.type) {
    case ActionType.Failed:
      state.failureCount = state.failureCount + 1;
      return state;
    case ActionType.Fetch:
      state.failureCount = 0;
      state.isFetching = true;
      state.isFetchingMore = action.isFetchingMore;
      state.status =
          state.data != null ? QueryStatus.Success : QueryStatus.Loading;
      return state;
    case ActionType.Success:
      state.canFetchMore = action.canFetchMore;
      state.data = action.data;
      state.error = null;
      state.failureCount = 0;
      state.isFetching = false;
      state.isFetchingMore = false;
      state.isInitialData = false;
      state.isInvalidated = false;
      state.status = QueryStatus.Success;
      state.updateCount = state.updateCount + 1;
      state.updatedAt =
          action.updatedAt ?? DateTime.now().millisecondsSinceEpoch;
      return state;
    case ActionType.Error:
      state.error = action.error;
      state.failureCount = state.failureCount + 1;
      state.isFetching = false;
      state.isFetchingMore = false;
      state.status = QueryStatus.Error;
      state.throwInErrorBoundary = true;
      state.updateCount = state.updateCount + 1;
      return state;
    case ActionType.Invalidate:
      state.isInvalidated = true;
      return state;
    default:
      return state;
  }
}
