// TYPES

import 'package:flutter_query/flutter_query.dart';
import 'package:flutter_query/src/notifyManager.dart';
import 'package:flutter_query/src/query.dart';
import 'package:flutter_query/src/query_observer.dart';
import 'package:flutter_query/src/types.dart';
import 'package:flutter_query/src/config.dart' as configNs;

class QueryCacheConfig {
  bool frozen;
  ReactQueryConfig defaultConfig;

  QueryCacheConfig({
    this.frozen = false,
  });
}

class ClearOptions {
  bool notify;
}

class PrefetchQueryOptions {
  bool force;
  bool throwOnError;
}

class RefetchQueriesOptions extends QueryPredicateOptions {
  bool throwOnError;
}

class InvalidateQueriesOptions extends RefetchQueriesOptions {
  bool refetchActive;
  bool refetchInactive;

  InvalidateQueriesOptions({
    this.refetchActive,
    this.refetchInactive,
  });
}

class QueryPredicateOptions {
  bool active;
  bool exact;
  bool stale;

  QueryPredicateOptions({
    this.active,
    this.exact,
    this.stale,
  });
}

// type QueryPredicate = QueryKey | QueryPredicateFn | true

// type QueryPredicateFn = (query: Query<unknown, unknown>) => boolean

class FetchQueryObjectConfig {
  dynamic queryKey;
  Function queryFn;
  BaseQueryConfig config;
}

class PrefetchQueryObjectConfig {
  dynamic queryKey;
  Function queryFn;
  BaseQueryConfig config;
  PrefetchQueryOptions options;
}

// TODO: Convert this
// interface QueryHashMap {
//   [hash: string]: Query<any, any>
// }

typedef QueryCacheListener = Function(
  QueryCache cache,
  Query query,
);

// CLASS

class QueryCache {
  int isFetching;

  // TODO: All instance properties below are private
  QueryCacheConfig config;
  List<QueryCacheListener> globalListeners;
  Map<String, Query> queries;
  List<Query> queriesArray;

  QueryCache([QueryCacheConfig config]) {
    this.config = config ?? QueryCacheConfig();
    this.globalListeners = [];
    this.queries = {};
    this.queriesArray = [];
    this.isFetching = 0;
  }

  notifyGlobalListeners([Query query]) {
    this.isFetching = this.getQueries().fold(
          0,
          (acc, q) => (q.state.isFetching ? acc + 1 : acc),
        );

    notifyManager.batch(() {
      this.globalListeners.forEach((listener) {
        notifyManager.schedule(() {
          listener(this, query);
        });
      });
    });
  }

  getDefaultConfig() {
    return this.config.defaultConfig;
  }

  ResolvedQueryConfig getResolvedQueryConfig(
    dynamic queryKey, [
    BaseQueryConfig config,
  ]) {
    return configNs.getResolvedQueryConfig(this, queryKey, null, config);
  }

  Function subscribe(QueryCacheListener listener) {
    this.globalListeners.add(listener);
    return () {
      this.globalListeners =
          this.globalListeners.where((x) => x != listener).toList();
    };
  }

  void clear([ClearOptions options]) {
    this.removeQueries();
    if (options?.notify ?? false) {
      this.notifyGlobalListeners();
    }
  }

  // TODO: Check this
  List<Query> getQueries([
    dynamic predicate,
    QueryPredicateOptions options,
  ]) {
    final anyKey = predicate == true || predicate == null;

    if (anyKey && options == null) {
      return this.queriesArray;
    }

    var predicateFn;

    if (predicate is Function) {
      predicateFn = predicate;
    } else {
      final exact = options?.exact ?? false;
      final active = options?.active;
      final stale = options?.stale;
      final resolvedConfig = this.getResolvedQueryConfig(predicate);

      predicateFn = (query) {
        // Check query key if needed
        if (!anyKey) {
          if (exact) {
            // Check if the query key matches exactly
            if (query.queryHash != resolvedConfig.queryHash) {
              return false;
            }
          } else {
            // TODO: This is a quick fix for handling strings but needs to change to support more than just key strings
            // Refer to react-query source
            if (query.queryKey.indexOf(predicate) == -1) {
              return false;
            }
          }
        }

        // Check active state if needed
        if (active is bool && query.isActive() != active) {
          return false;
        }

        // Check stale state if needed
        if (stale is bool && query.isStale() != stale) {
          return false;
        }

        return true;
      };
    }

    return this.queriesArray.where(predicateFn).toList();
  }

  Query getQuery(dynamic predicate) {
    return this.getQueries(predicate, QueryPredicateOptions(exact: true))[0];
  }

  Query getQueryByHash(
    String queryHash,
  ) {
    return this.queries[queryHash];
  }

  dynamic getQueryData(Function predicate) {
    return this.getQuery(predicate)?.state?.data;
  }

  void removeQuery(Query query) {
    if (this.queries[query.queryHash] != null) {
      query.destroy();
      this.queries[query.queryHash] = null;
      this.queriesArray = this.queriesArray.where((x) => x != query).toList();
      this.notifyGlobalListeners(query);
    }
  }

  void removeQueries([
    dynamic predicate,
    QueryPredicateOptions options,
  ]) {
    getQueries(predicate, options).forEach((query) => this.removeQuery(query));
  }

  void cancelQueries([
    dynamic predicate,
    QueryPredicateOptions options,
  ]) {
    this.getQueries(predicate, options).forEach((query) {
      query.cancel();
    });
  }

  /**
   * @return Promise resolving to an array with the invalidated queries.
   */
  Future<List<Query>> invalidateQueries(
    dynamic predicate, [
    InvalidateQueriesOptions options,
  ]) {
    final queries = this.getQueries(predicate, options);

    notifyManager.batch(() {
      queries.forEach((query) => query.invalidate());
    });

    final refetchActive = options?.refetchActive ?? true;
    final refetchInactive = options?.refetchInactive ?? false;

    if (!refetchInactive && !refetchActive) {
      return Future.value(queries);
    }

    // TODO: Convert this
    // final refetchOptions: RefetchQueriesOptions = { ...options }
    final refetchOptions = options ?? RefetchQueriesOptions();

    if (refetchActive && !refetchInactive) {
      refetchOptions.active = true;
    } else if (refetchInactive && !refetchActive) {
      refetchOptions.active = false;
    }

    var promise = this.refetchQueries(predicate, refetchOptions);

    if (!(options?.throwOnError ?? false)) {
      promise = promise.catchError((_) => queries);
    }

    return promise.then((_) => queries);
  }

  /**
   * @return Promise resolving to an array with the refetched queries.
   */
  Future<List<Query>> refetchQueries(
    dynamic predicate, [
    RefetchQueriesOptions options,
  ]) {
    List<Future<Query>> promises = [];

    notifyManager.batch(() {
      this.getQueries(predicate, options).forEach((query) {
        var promise = query.fetch().then((_) => query);

        if (!(options?.throwOnError ?? false)) {
          promise = promise.catchError((_) => query);
        }

        promises.add(promise);
      });
    });

    return Future.wait(promises);
  }

  void resetErrorBoundaries() {
    this
        .getQueries()
        .forEach((query) => query.state.throwInErrorBoundary = false);
  }

  Query buildQuery(
    dynamic queryKey,
    BaseQueryConfig config,
  ) {
    final resolvedConfig = this.getResolvedQueryConfig(queryKey, config);
    var query = this.getQueryByHash(resolvedConfig.queryHash);

    if (query == null) {
      query = this.createQuery(resolvedConfig);
    }

    return query;
  }

  Query createQuery(
    BaseQueryConfig config,
  ) {
    final query = Query(config);

    // A frozen cache does not add new queries to the cache
    if (!this.config.frozen) {
      this.queries[query.queryHash] = query;
      this.queriesArray.add(query);
      this.notifyGlobalListeners(query);
    }

    return query;
  }

  // Notice: Removed all fetchQuery and friends function definitions due to dart not supporting overloading

  // Implementation
  Future fetchQuery(
    dynamic queryKey, {
    Function queryFn,
    PrefetchQueryOptions options,
    ResolvedQueryConfig queryConfig,
    FetchQueryObjectConfig config,
  }) {
    // TODO: Refer to source code, this is missing stuff
    final resolvedConfig = getResolvedQueryConfig(queryKey);
    var query = this.getQueryByHash(queryKey);

    if (query == null) {
      query = this.createQuery(resolvedConfig);
    }

    if (!query.isStaleByTime(queryConfig.staleTime)) {
      return Future.value(query.state.data);
    }

    return query.fetch(null, resolvedConfig);
  }

  // Implementation
  Future prefetchQuery(
    dynamic queryKey, {
    Function queryFn,
    PrefetchQueryOptions options,
    ResolvedQueryConfig queryConfig,
    PrefetchQueryObjectConfig config,
  }) {
    if ((options?.force ?? false) && queryConfig != null) {
      queryConfig.staleTime = 0;
    }

    var promise = this.fetchQuery(queryKey, queryFn: queryFn);

    if (!(options?.throwOnError ?? false)) {
      promise = promise.catchError((err) {});
    }

    return Future.value();
  }

  // Implementation
  QueryObserver watchQuery(
    dynamic queryKey, {
    BaseQueryConfig queryConfig,
    Function queryFn,
  }) {
    final resolvedConfig = this.getResolvedQueryConfig(queryKey, queryConfig);
    return QueryObserver(resolvedConfig);
  }

  setQueryData(
    dynamic queryKey,
    Function updater,
    BaseQueryConfig config,
  ) {
    this.buildQuery(queryKey, config).setData(updater);
  }
}
