import 'dart:async';
import 'dart:math';

import 'package:flutter_query/src/config.dart';
import 'package:flutter_query/src/notifyManager.dart';
import 'package:flutter_query/src/query.dart';
import 'package:flutter_query/src/types.dart';
import 'package:flutter_query/src/utils.dart';

import 'utils.dart';

typedef UpdateListener(QueryResult result);

class NotifyOptions {
  bool globalListeners;
  bool listener;
  bool onError;
  bool onSuccess;

  // TODO: Double check these defaults
  NotifyOptions({
    this.onError = false,
    this.onSuccess = false,
    this.listener = false,
    this.globalListeners = false,
  });
}

class QueryObserver {
  ResolvedQueryConfig config;

  // TODO: Instance properties below are private
  Query currentQuery;
  QueryResult currentResult;
  QueryResult previousQueryResult;
  UpdateListener listener;
  bool isStale;
  int initialUpdateCount;
  int staleTimeoutId;

  StreamSubscription _staleSubscription;
  StreamSubscription _refetchSubscription;

  QueryObserver(QueryObserverConfig config) {
    this.config = config;
    this.isStale = true;
    this.initialUpdateCount = 0;

    // Subscribe to the query
    this.updateQuery();
  }

  Function subscribe(UpdateListener listener) {
    this.listener = listener ?? noop;
    this.currentQuery.subscribeObserver(this);

    if (this.config.enabled &&
        (this.config.forceFetchOnMount ||
            this.config.refetchOnMount == 'always')) {
      this.fetch();
    } else {
      this.optionalFetch();
    }

    this.updateTimers();

    return this.unsubscribe;
  }

  void unsubscribe() {
    this.listener = null;
    this.clearTimers();
    this.currentQuery.unsubscribeObserver(this);
  }

  void updateConfig(
    QueryObserverConfig config,
  ) {
    final prevConfig = this.config;
    final prevQuery = this.currentQuery;

    this.config = isResolvedQueryConfig(config)
        ? config
        : this
            .config
            .queryCache
            .getResolvedQueryConfig(this.config.queryKey, config);

    this.updateQuery();

    // Take no further actions if there is no subscriber
    if (this.listener == null) {
      return;
    }

    // If we subscribed to a new query, optionally fetch and update refetch
    if (this.currentQuery != prevQuery) {
      this.optionalFetch();
      this.updateTimers();
      return;
    }

    // Optionally fetch if the query became enabled
    if (config.enabled && !prevConfig.enabled) {
      this.optionalFetch();
    }

    // Update stale interval if needed
    if (config.enabled != prevConfig.enabled ||
        config.staleTime != prevConfig.staleTime) {
      this.updateStaleTimeout();
    }

    // Update refetch interval if needed
    if (config.enabled != prevConfig.enabled ||
        config.refetchInterval != prevConfig.refetchInterval) {
      this.updateRefetchInterval();
    }
  }

  Query getCurrentQuery() {
    return this.currentQuery;
  }

  QueryResult getCurrentResult() {
    return this.currentResult;
  }

  /**
   * @deprecated
   */
  void clear() {
    this.remove();
  }

  void remove() {
    this.currentQuery.remove();
  }

  Future refetch([RefetchOptions options]) {
    return this.currentQuery.refetch(options, this.config);
  }

  Future fetchMore([
    dynamic fetchMoreVariable,
    FetchMoreOptions options,
  ]) {
    return this
        .currentQuery
        .fetchMore(fetchMoreVariable, options, this.config)
        .catchError((err) {});
  }

  Future fetch() {
    // Never try to fetch if no query function has been set
    if (this.config.queryFn == null) {
      return Future.sync(() => this.currentResult.data);
    }

    return this.currentQuery.fetch(null, this.config).catchError((err) {});
  }

  // private
  void optionalFetch() {
    if (this.config.enabled && // Only fetch if enabled
        this.isStale && // Only fetch if stale
        ((this.config.refetchOnMount != null &&
                this.config.refetchOnMount != false) ||
            this.currentQuery.observers.length == 1)) {
      this.fetch();
    }
  }

  // private
  void updateStaleTimeout() {
    this.clearStaleTimeout();

    if (this.isStale || !isValidTimeout(this.config.staleTime)) {
      return;
    }

    // TODO: Make sure this is correct
    final timeElapsed =
        DateTime.now().millisecondsSinceEpoch - this.currentResult.updatedAt;
    final timeUntilStale = this.config.staleTime - timeElapsed + 1;
    final timeout = max(timeUntilStale, 0);

    _staleSubscription = Future.delayed(Duration(
      milliseconds: timeout,
    )).asStream().listen((_) {
      if (!this.isStale) {
        this.isStale = true;
        this.updateResult();
        this.notify(NotifyOptions(
          listener: true,
          globalListeners: true,
        ));
      }
    });
  }

  // private
  void updateRefetchInterval() {
    this.clearRefetchInterval();

    if (!this.config.enabled || !isValidTimeout(config.refetchInterval)) {
      return;
    }

    if (config.refetchInterval != null) {
      _refetchSubscription = Stream.periodic(
        Duration(
          milliseconds: config.refetchInterval,
        ),
      ).listen(
        (event) {
          if (this.config.refetchIntervalInBackground) {
            this.fetch();
          }
        },
      );
    }
  }

  void updateTimers() {
    this.updateStaleTimeout();
    this.updateRefetchInterval();
  }

  void clearTimers() {
    this.clearStaleTimeout();
    this.clearRefetchInterval();
  }

  // private
  void clearStaleTimeout() {
    if (_staleSubscription != null) {
      _staleSubscription.cancel();
      _staleSubscription = null;
    }
  }

  // private
  void clearRefetchInterval() {
    if (_refetchSubscription != null) {
      _refetchSubscription.cancel();
      _refetchSubscription = null;
    }
  }

  // private
  void updateResult() {
    final state = currentQuery.state;
    var data = state.data;
    var status = state.status;
    var updatedAt = state.updatedAt;
    var isPreviousData = false;

    // Keep previous data if needed
    if (this.config.keepPreviousData &&
        state.isInitialData &&
        (this.previousQueryResult?.isSuccess ?? false)) {
      data = this.previousQueryResult.data;
      updatedAt = this.previousQueryResult.updatedAt;
      status = this.previousQueryResult.status;
      isPreviousData = true;
    }

    this.currentResult = QueryResult(
      status: status,
      canFetchMore: state.canFetchMore,
      clear: this.remove,
      data: data,
      error: state.error,
      failureCount: state.failureCount,
      fetchMore: this.fetchMore,
      isFetched: state.updateCount > 0,
      isFetchedAfterMount: state.updateCount > this.initialUpdateCount,
      isFetching: state.isFetching,
      isFetchingMore: state.isFetchingMore,
      isInitialData: state.isInitialData,
      isPreviousData: isPreviousData,
      isStale: this.isStale ?? false,
      refetch: this.refetch,
      remove: this.remove,
      updatedAt: updatedAt,
    );
  }

  // private
  void updateQuery() {
    final config = this.config;
    final prevQuery = this.currentQuery;

    var query = config.queryCache.getQueryByHash(config.queryHash);

    if (query == null) {
      query = config.queryCache.createQuery(config);
    }

    if (query == prevQuery) {
      return;
    }

    this.previousQueryResult = this.currentResult;
    this.currentQuery = query;
    this.initialUpdateCount = query.state.updateCount;

    // Update stale state on query switch
    if (query.state.isInitialData) {
      if (config.keepPreviousData && prevQuery != null) {
        this.isStale = true;
      } else if (config.initialStale is Function) {
        this.isStale = config.initialStale();
      } else if (config.initialStale is bool) {
        this.isStale = config.initialStale;
      } else {
        this.isStale = query.state.data == null;
      }
    } else {
      this.isStale = query.isStaleByTime(config.staleTime);
    }

    this.updateResult();
    if (this.listener != null) {
      prevQuery?.unsubscribeObserver(this);
      this.currentQuery.subscribeObserver(this);
    }
  }

  void onQueryUpdate(Action action) {
    final type = action.type;

    // Update stale state on success, error or invalidation
    if (type == ActionType.Success ||
        type == ActionType.Error ||
        type == ActionType.Invalidate) {
      this.isStale = this.currentQuery.isStaleByTime(config.staleTime);
    }

    // Store current result and get new result
    final prevResult = this.currentResult;
    this.updateResult();
    final currentResult = this.currentResult;

    // Update timers on success, error or invalidation
    if (type == ActionType.Success ||
        type == ActionType.Error ||
        type == ActionType.Invalidate) {
      this.updateTimers();
    }

    // Do not notify if the query was invalidated but the stale state did not changed
    if (type == ActionType.Invalidate &&
        currentResult.isStale == prevResult.isStale) {
      return;
    }
    // Determine which callbacks to trigger
    final notifyOptions = NotifyOptions();

    if (type == ActionType.Success) {
      notifyOptions.onSuccess = true;
    } else if (type == ActionType.Error) {
      notifyOptions.onError = true;
    }

    if (
        // Always notify if notifyOnStatusChange is set
        config.notifyOnStatusChange ||
            // Otherwise only notify on data or error change
            currentResult.data != prevResult.data ||
            currentResult.error != prevResult.error) {
      notifyOptions.listener = true;
    }

    this.notify(notifyOptions);
  }

  // private
  void notify(NotifyOptions options) {
    final onSuccess = config.onSuccess;
    final onSettled = config.onSettled;
    final onError = config.onError;

    notifyManager.batch(() {
      // First trigger the configuration callbacks
      if (options.onSuccess) {
        if (onSuccess != null) {
          notifyManager.schedule(() {
            onSuccess(currentResult.data);
          });
        }
        if (onSettled != null) {
          notifyManager.schedule(() {
            onSettled(currentResult.data, null);
          });
        }
      } else if (options.onError) {
        if (onError != null) {
          notifyManager.schedule(() {
            onError(currentResult.error);
          });
        }
        if (onSettled != null) {
          notifyManager.schedule(() {
            onSettled(null, currentResult.error);
          });
        }
      }

      // Then trigger the listener
      if (options.listener && listener != null) {
        notifyManager.schedule(() {
          listener(currentResult);
        });
      }

      // Then the global listeners
      if (options.globalListeners) {
        config.queryCache.notifyGlobalListeners(currentQuery);
      }
    });
  }
}
