import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:flutter_query/flutter_query.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:provider/provider.dart';

void main() {
  group('useQuery', () {
    testWidgets('should return the correct states for a successful query',
        (tester) async {
      final key = 'testKey';
      List<QueryResult> states = [];

      await tester.pumpFrames(
        QueryCacheProvider(
          child: HookBuilder(
            builder: (context) {
              final state = useQuery(key, () => Future.value('test'));
              states.add(state);

              return Container();
            },
          ),
        ),
        Duration(seconds: 5),
      );

      expect(states[0].data, null);
      expect(states[0].canFetchMore, false);

      expect(states[1].data, 'test');
      expect(states[1].failureCount, 0);
    });

    testWidgets(
      'should return the correct states for an unsuccessful query',
      (tester) async {
        final key = 'testKey';
        List<QueryResult> states = [];

        await tester.pumpFrames(
          QueryCacheProvider(
            child: HookBuilder(
              builder: (context) {
                final state = useQuery(
                  key,
                  () =>
                      Future.delayed(Duration(milliseconds: 300)).then((value) {
                    throw 'rejected';
                  }),
                  retry: 1,
                  retryDelay: 1,
                );

                states.add(state);

                return Container();
              },
            ),
          ),
          Duration(seconds: 5),
        );

        expect(states[0].failureCount, 0);
        expect(states[0].canFetchMore, false);
        expect(states[0].data, null);
        expect(states[0].error, null);
        expect(states[0].isError, false);
        expect(states[0].isFetched, false);
        expect(states[0].isFetchedAfterMount, false);
        expect(states[0].isFetching, true);
        expect(states[0].isFetchingMore, false);
        expect(states[0].isIdle, false);
        expect(states[0].isInitialData, true);
        expect(states[0].isLoading, true);
        expect(states[0].isPreviousData, false);
        expect(states[0].isStale, true);
        expect(states[0].isSuccess, false);
        expect(states[0].status, QueryStatus.Loading);

        expect(states[1].failureCount, 1);
        expect(states[1].canFetchMore, false);
        expect(states[1].data, null);
        expect(states[1].error, null);
        expect(states[1].isError, false);
        expect(states[1].isFetched, false);
        expect(states[1].isFetchedAfterMount, false);
        expect(states[1].isFetching, true);
        expect(states[1].isFetchingMore, false);
        expect(states[1].isIdle, false);
        expect(states[1].isInitialData, true);
        expect(states[1].isLoading, true);
        expect(states[1].isPreviousData, false);
        expect(states[1].isStale, true);
        expect(states[1].isSuccess, false);
        expect(states[1].status, QueryStatus.Loading);

        expect(states[2].failureCount, 2);
        expect(states[2].canFetchMore, false);
        expect(states[2].data, null);
        expect(states[2].error, 'rejected');
        expect(states[2].isError, true);
        expect(states[2].isFetched, true);
        expect(states[2].isFetchedAfterMount, true);
        expect(states[2].isFetching, false);
        expect(states[2].isFetchingMore, false);
        expect(states[2].isIdle, false);
        expect(states[2].isInitialData, true);
        expect(states[2].isLoading, false);
        expect(states[2].isPreviousData, false);
        expect(states[2].isStale, true);
        expect(states[2].isSuccess, false);
        expect(states[2].status, QueryStatus.Error);
      },
    );

    testWidgets(
      'should update query stale state when invalidated with invalidateQueries',
      (tester) async {
        final queryCache = QueryCache();
        List<QueryResult> states = [];

        await tester.pumpFrames(
          Provider.value(
            value: queryCache,
            child: HookBuilder(
              builder: (context) {
                final state = useQuery(
                  'test',
                  () => Future.delayed(Duration(milliseconds: 20))
                      .then((_) => 'data'),
                  staleTime: double.maxFinite,
                );

                states.add(state);

                useEffect(() {
                  return Timer(Duration(milliseconds: 10), () {
                    queryCache.invalidateQueries(
                      'test',
                      InvalidateQueriesOptions(
                        refetchActive: false,
                        refetchInactive: false,
                      ),
                    );
                  }).cancel;
                });

                return Container();
              },
            ),
          ),
          Duration(milliseconds: 100),
        );

        await tester.pumpAndSettle(Duration(milliseconds: 100));

        expect(states.length, 3);
      },
    );
  });

  // testWidgets('useInfiniteQuery', (tester) async {
  //   var passed = false;

  //   await tester.pumpFrames(
  //     QueryCacheProvider(
  //       child: HookBuilder(
  //         builder: (context) {
  //           final result = useInfiniteQuery<List<String>>(
  //             'data',
  //             (page) {
  //               if (page == null) {
  //                 return Future.value([]);
  //               }
  //               final value =
  //                   page > 2 ? List<String>.empty() : [page.toString()];
  //               return Future<List<String>>.value(value);
  //             },
  //             getFetchMore: (latestPage, allPages) {
  //               if (latestPage is List && latestPage.length == 0) {
  //                 return false;
  //               }

  //               return (allPages ?? []).length + 1;
  //             },
  //           );

  //           if (!result.canFetchMore && !result.isLoading) {
  //             passed = true;
  //           }
  //           if (result.isSuccess && result.canFetchMore) {
  //             result.fetchMore();
  //           }
  //           return Container();
  //         },
  //       ),
  //     ),
  //     Duration(seconds: 5),
  //   );

  //   expect(passed, true);
  // });

  // testWidgets('useInfiniteQuery invalidate', (tester) async {
  //   var passed = false;

  //   await tester.pumpFrames(
  //     Provider.value(
  //       value: queryCache,
  //       builder: (_, child) {
  //         return HookBuilder(
  //           builder: (context) {
  //             final result = useInfiniteQuery<List<String>>(
  //               'data/test',
  //               (page) {
  //                 final value = (page ?? 0) > 2
  //                     ? List<String>.empty()
  //                     : [(page ?? 0).toString()];
  //                 return Future<List<String>>.value(value);
  //               },
  //               getFetchMore: (latestPage, allPages) {
  //                 if (latestPage is List && latestPage.length == 0) {
  //                   return false;
  //                 }

  //                 return (allPages ?? []).length + 1;
  //               },
  //             );

  //             if (result.isSuccess && result.canFetchMore) {
  //               result.fetchMore();
  //             }

  //             if (!result.canFetchMore && result.isSuccess && !passed) {
  //               passed = true;
  //               Provider.of<QueryCache>(context).invalidateQueries('data');
  //             }
  //             return Container();
  //           },
  //         );
  //       },
  //     ),
  //     Duration(seconds: 5),
  //   );

  //   expect(passed, true);
  // });

  // testWidgets('useQuery refetch', (tester) async {
  //   int hits = 0;

  //   await tester.pumpFrames(
  //     QueryCacheProvider(
  //       child: HookBuilder(
  //         builder: (context) {
  //           final result = useQuery(
  //             'data',
  //             () => Future.value('Hello!'),
  //             refetchInterval: 1000,
  //             refetchIntervalInBackground: true,
  //             onSuccess: (data) {
  //               print('Success!');
  //             },
  //             onError: (err) {
  //               print('Shouldnt hit here!');
  //             },
  //           );

  //           if (result.isSuccess && result.data == 'Hello!') {
  //             hits++;
  //           }
  //           return Container();
  //         },
  //       ),
  //     ),
  //     Duration(seconds: 2),
  //   );

  //   expect(hits, 2);
  // });
}
